'''
Functions for helping curate BSE basis set data
'''

from .printing import *
from .compare import *
from .compare_ref import *
from .metadata import *
from .read_g94 import *
from .read_turbomole import *
